
const.b1<-2
const.b2<-2
const.b3<-2
const.b.vec<-c(const.b1, const.b2, const.b3)


# para=(pi1, pi2, 
#       muc1, tau.c1, r.c1,
#       delta.n1, tau.n1, r.n1,
#       mu.2, tau.2, r.2,
#       mu.c3, tau.c3, r.c3,
#       delta.n3, tau.n3, r.n3)
# total  17 parameters
 
"paraEst" <-
function(X, paraIni, memSubjects, maxFlag=TRUE, thrshPostProb=0.50, 
         geneNames=NULL, ITMAX=100, eps=1.0e-03, quiet=TRUE)
{
  # check if parameters are in appropriate ranges
  checkPara(paraIni, eps)

  sumb<-sum(const.b.vec)
  const.b1.b2<-c(const.b1, const.b2)

  n<-ncol(X) # number of patients/subjects
  nc<-sum(memSubjects==1)  # number of cases
  nn<-sum(memSubjects==0)  # number of controls
  if(nc>=n){
    stop("Number of cases >= Total number of patients!\n")
  }
  if(nn>=n){
    stop("Number of controls >= Total number of patients!\n")
  }

  # start em algorithm
  Psi.m<-paraIni
  #cat("Psi.m>>\n"); print(Psi.m); cat("\n");
  nGenes<-nrow(X)

  ##############################

  xcMat<-X[,memSubjects==1,drop=FALSE]
  xnMat<-X[,memSubjects==0,drop=FALSE]

  xcTxc<-apply(xcMat, 1, function(x) {sum(x^2)})
  xcT1<-apply(xcMat, 1, function(x) {sum(x)})

  xnTxn<-apply(xnMat, 1, function(x) {sum(x^2)})
  xnT1<-apply(xnMat, 1, function(x) {sum(x)})

  xTx<-apply(X, 1, function(x) {sum(x^2)})
  xT1<-apply(X, 1, function(x) {sum(x)})


  loop<-0
  while(1)
  { 
    if(!quiet)
    {
      cat("************\n")
      cat("loop=", loop, "\n")
    }

    # E-step
    # obtain weights w_{ij}(\Psi^{(m)}), i.e. E(Z_{ij} | x_i, Psi^{(m)})
    wiMat<-t(apply(X, 1, wiFun, Psi.m=Psi.m, memSubjects=memSubjects, eps=eps))

    w1<-as.numeric(wiMat[,1])
    w2<-as.numeric(wiMat[,2])
    w3<-as.numeric(wiMat[,3])
    sumw1<-sum(w1)
    sumw2<-sum(w2)
    sumw3<-sum(w3)

    if(!quiet)
    { 
      cat("sumw1=", sumw1, " sumw2=", sumw2, " sumw3=", sumw3, "\n")    
    }

    sumw1xcTxc<-sum(w1*xcTxc)
    sumw1xcT1<-sum(w1*xcT1)
    sumw1xcT1.sq<-sum(w1*xcT1^2)
 
    sumw1xnTxn<-sum(w1*xnTxn)
    sumw1xnT1<-sum(w1*xnT1)
    sumw1xnT1.sq<-sum(w1*xnT1^2)
 
    sumw2xTx<-sum(w2*xTx)
    sumw2xT1<-sum(w2*xT1)
    sumw2xT1.sq<-sum(w2*xT1^2)
 
    sumw3xcTxc<-sum(w3*xcTxc)
    sumw3xcT1<-sum(w3*xcT1)
    sumw3xcT1.sq<-sum(w3*xcT1^2)
 
    sumw3xnTxn<-sum(w3*xnTxn)
    sumw3xnT1<-sum(w3*xnT1)
    sumw3xnT1.sq<-sum(w3*xnT1^2)
 
    # M-step
    # mixture proportions
    piVec.new<-(c(sumw1, sumw2)+const.b1.b2-1)/(nGenes+sumb-3)
    names(piVec.new)<-c("pi.1", "pi.2")

    thetaIni<-Psi.m[-c(1:2)]
    nTheta<-length(thetaIni)
    lower<-rep(-Inf, nTheta)
    upper<-rep(Inf, nTheta)
    res<-optim(par=thetaIni, fn=negQFunc, 
          nc=nc, nn=nn, n=n, xcMat=xcMat, xnMat=xnMat, 
          xcTxc=xcTxc, xnTxn=xnTxn, xTx=xTx, xT1=xT1,
          sumw1=sumw1, sumw2=sumw2, sumw3=sumw3, 
          sumw1xcTxc=sumw1xcTxc, sumw1xcT1=sumw1xcT1, sumw1xcT1.sq=sumw1xcT1.sq,
          sumw1xnTxn=sumw1xnTxn, sumw1xnT1=sumw1xnT1, sumw1xnT1.sq=sumw1xnT1.sq,
          sumw2xTx=sumw2xTx, sumw2xT1=sumw2xT1, sumw2xT1.sq=sumw2xT1.sq,
          sumw3xcTxc=sumw3xcTxc, sumw3xcT1=sumw3xcT1, sumw3xcT1.sq=sumw3xcT1.sq,
          sumw3xnTxn=sumw3xnTxn, sumw3xnT1=sumw3xnT1, sumw3xnT1.sq=sumw3xnT1.sq,
          method = "L-BFGS-B",
          lower = lower, upper = upper)

    if(res$convergence)
    {
      cat("convergence=", res$convergence, "\n")
      cat("message=", res$message, "\n")
      cat("number of calls to fn and gr>>\n"); print(res$counts); cat("\n");
      cat("thetaIni>>\n"); print(thetaIni); cat("\n");
    }
 
    negQ<-res$value
    theta<-res$par

    # update parameters
    Psi.m.new<-c(piVec.new, theta)
    names(Psi.m.new)<-paraNamesRP

    if(!quiet)
    { cat("Psi.m.new>>\n"); print(round(Psi.m.new,3)); cat("\n") }
      
    err.len<-sum(abs(Psi.m.new-Psi.m)<eps)
    if(err.len==length(Psi.m))
    {
      # the following statement is to 
      # make sure eta.new is an update from current wMat
      # so that d Q / d eta = 0
      Psi.m<-Psi.m.new
      break
    }

    Psi.m<-Psi.m.new
    loop<-loop+1

    if(loop>ITMAX)
    { cat("*********************\n") 
      cat("Warning! Number of looping (= ", loop, 
        ") exceeds ITMAX (=", ITMAX, ")!\n")
      cat("EM algorithm did not converge!\n")
      cat("*********************\n") 
      break
    }
  }

  
  if(!quiet)
  { cat("Total iterations for EM algorithm=", loop, "\n") }

  # gene cluster membership
  memMat<-t(apply(X, 1, wiFun, Psi.m=Psi.m, memSubjects=memSubjects, eps=eps))
  colnames(memMat)<-c("cluster1", "cluster2", "cluster3")
  rownames(memMat)<-geneNames

  # update gene cluster membership
  memGenes<-apply(memMat, 1, maxPosFun, maxFlag=maxFlag, 
                  thrshPostProb=thrshPostProb)
  sumw1<-mean(memMat[,1])
  sumw2<-mean(memMat[,2])
  sumw3<-mean(memMat[,3])

  sumw1xcTxc<-sum(w1*xcTxc)
  sumw1xcT1<-sum(w1*xcT1)
  sumw1xcT1.sq<-sum(w1*xcT1^2)
 
  sumw1xnTxn<-sum(w1*xnTxn)
  sumw1xnT1<-sum(w1*xnT1)
  sumw1xnT1.sq<-sum(w1*xnT1^2)
 
  sumw2xTx<-sum(w2*xTx)
  sumw2xT1<-sum(w2*xT1)
  sumw2xT1.sq<-sum(w2*xT1^2)
 
  sumw3xcTxc<-sum(w3*xcTxc)
  sumw3xcT1<-sum(w3*xcT1)
  sumw3xcT1.sq<-sum(w3*xcT1^2)
 
  sumw3xnTxn<-sum(w3*xnTxn)
  sumw3xnT1<-sum(w3*xnT1)
  sumw3xnT1.sq<-sum(w3*xnT1^2)
 
  llkh<- -negQFunc(Psi.m, X, nc, nn, n,
    xcMat, xnMat, xcTxc, xnTxn, xTx, xT1,
    sumw1, sumw2, sumw3, 
    sumw1xcTxc, sumw1xcT1, sumw1xcT1.sq,
    sumw1xnTxn, sumw1xnT1, sumw1xnT1.sq,
    sumw2xTx, sumw2xT1, sumw2xT1.sq,
    sumw3xcTxc, sumw3xcT1, sumw3xcT1.sq,
    sumw3xnTxn, sumw3xnT1, sumw3xnT1.sq)

  memGenes2<-rep(1, nGenes)
  memGenes2[memGenes==2]<-0

  if(sum(is.null(geneNames)))
  {
    geneNames<-paste("gene", 1:nGenes, sep="")
  } 

  names(Psi.m)<-paraNamesRP
  names(memGenes)<-geneNames
  names(memGenes2)<-geneNames

  invisible(list(para=Psi.m, llkh=llkh, memGenes=memGenes, 
                 memGenes2=memGenes2, memMat=memMat, loop=loop))
}

# find the position of the maximum element of 'x'
maxPosFun<-function(x, maxFlag=TRUE, thrshPostProb=0.50)
{
  if(maxFlag)
  { pos<-which(x==max(x))
    pos<-pos[1]
    return(pos)
  }
    
  pos<-which(x>thrshPostProb)
  len<-length(pos)
  if(len==0)
  { 
    return(2) 
  } # non-differentially expressed

  pos<-which(x==max(x))

  return(pos)
}


